import spacy
import pandas as pd
import numpy as np
import argparse
from tqdm import tqdm
from .content_preservation import (
    corpus_bleu,
    cosine_similarity,
    word_overlap
)
from .language_quality import sentence_ppl
from .style_transfer_accuracy import sta_score
from .przetak_wrapper import PrzetakWrapper
from .gpt2_wrapper import GPT2Wrapper
from nltk.translate.bleu_score import sentence_bleu
from sentence_transformers import SentenceTransformer


GOPATH = "/Users/cezary/go"


gpt2 = GPT2Wrapper()
spacy = spacy.load("pl_core_news_lg")
przetak = PrzetakWrapper(GOPATH, spacy)
sbert = SentenceTransformer(
    'sentence-transformers/distiluse-base-multilingual-cased-v1'
)


def sentence_scores(original_sentence, transfered_sentence):
    sta = sta_score(transfered_sentence, przetak)
    bleu = sentence_bleu([original_sentence], transfered_sentence)
    cs = cosine_similarity(original_sentence, transfered_sentence, sbert)
    wo = word_overlap(original_sentence, transfered_sentence, spacy)

    if transfered_sentence != '':
        ppl = sentence_ppl(transfered_sentence, gpt2)
    else:
        ppl = np.nan

    if (not np.isnan(ppl)) and ppl != 0:
        ppl_inv = 1 / ppl
    else:
        ppl_inv = np.nan

    gm = (100*max(0, cs) * 100*max(0, sta) * max(0, ppl_inv)) ** (1/3)

    return dict(
        original=original_sentence,
        transfered=transfered_sentence,
        sta=np.float32(sta),
        bleu=np.float32(bleu),
        cs=np.float32(cs),
        wo=np.float32(wo),
        ppl=np.float32(ppl),
        gm=np.float32(gm)
    )


def corpus_scores(original_sentences, transfered_sentences):
    out = pd.DataFrame(
        [],
        columns=list(sentence_scores(
            original_sentences[0],
            transfered_sentences[0]
        ).keys())
    )

    for original, transfered in zip(original_sentences, transfered_sentences):
        out.loc[len(out), :] = list(
            sentence_scores(original, transfered).values()
        )

    return out


def evaluate(df):
    models = [col for col in df.columns
              if col != 'text']

    original_sentences = [sent for sent in df['text']]

    bleus = dict()

    for i in tqdm(range(len(models)), total=len(models)):
        model = models[i]
        transfered_sentences = [sent for sent in df[model].fillna("")]

        stats = corpus_scores(original_sentences, transfered_sentences)
        bleu = corpus_bleu(original_sentences, transfered_sentences)

        stats['model'] = model
        bleus[model] = bleu

        if i == 0:
            out = stats
        else:
            out = out.append(stats, ignore_index=True)

    for col in [column for column in out.columns
                if column not in ['original', 'transfered', 'model']]:
        out[col] = np.float32(out[col])

    out = out.drop(['original', 'transfered'], axis=1) \
             .groupby(['model']) \
             .mean() \
             .reset_index()
    out['bleu'] = out.model.apply(lambda x: bleus[x])
    return out


def get_delete_baseline(text):
    return przetak.delete(text)
